/*
 * qemu_hugepages.h: QEMU memory control
 *
 * Copyright (c) 2020 Oracle and/or its affiliates
 *
 * Autonomous memoryBacking nr_hugepage reservations under control of libvirt
 */

#pragma once

#ifndef LIBVIRT_QEMU_HUGEPAGES_H
# define LIBVIRT_QEMU_HUGEPAGES_H

# include "qemu_conf.h"
# include "qemu_domain.h"

# define PAGE_4K         0
# define PAGE_2M         1
# define PAGE_1G         2

# define MIN_HUGEPAGE_SPAREPAGES 512

int qemuMemAcquireHostHugepages(virQEMUDriverPtr driver,
                                virDomainDefPtr def,
                                virQEMUDriverConfigPtr cfg,
                                unsigned long long size,
                                int targetNode)
    ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2)
    ATTRIBUTE_RETURN_CHECK;

int qemuMemReleaseHostHugepages(virQEMUDriverPtr driver,
                                virDomainDefPtr def,
                                virDomainMemoryDefPtr mem)
    ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2)
    ATTRIBUTE_RETURN_CHECK;

typedef struct _qemuHostNUMACellPageInfo qemuHostNUMACellPageInfo;
typedef qemuHostNUMACellPageInfo *qemuHostNUMACellPageInfoPtr;

struct _qemuHostNUMACellPageInfo {
    unsigned int size;          /* page size in kibibytes */
    unsigned long long avail;   /* size of the pool */
    unsigned long long free;    /* number of free pages in the pool */
};

typedef struct _qemuGuestNUMACellReq qemuGuestNUMACellReq;
typedef qemuGuestNUMACellReq *qemuGuestNUMACellReqPtr;

struct _qemuGuestNUMACellReq {
    unsigned long long nr_pages; /* Guest node required cell #pages */
    unsigned long long nr_free;  /* Host node free pages in the pool */
};

#endif /* LIBVIRT_QEMU_HUGEPAGES_H */
